@@ -41,7 +41,9 @@ class AdaptiveParallelismStrategy:
         """Set the OllamaPool after initialization."""
         self.pool = pool
 
-    def should_parallelize(self, batch_size: int, model_name: Optional[str] = None) -> Tuple[bool, Dict]:
+    def should_parallelize(
+        self, batch_size: int, model_name: Optional[str] = None
+    ) -> Tuple[bool, Dict]:
         """
         Decide whether to parallelize based on cluster state.
 
@@ -54,7 +56,10 @@ class AdaptiveParallelismStrategy:
         """
         if not self.pool:
             logger.warning("No OllamaPool set, defaulting to parallel")
-            return True, {"reason": "no_pool", "detail": "No pool configured, using parallel by default"}
+            return True, {
+                "reason": "no_pool",
+                "detail": "No pool configured, using parallel by default",
+            }
 
         nodes = self.pool.nodes
         node_performance = self.pool.stats.get("node_performance", {})
@@ -70,7 +75,7 @@ class AdaptiveParallelismStrategy:
         # Calculate performance metrics
         node_speeds = []
 
-        for node in nodes:
+        for i, node in enumerate(nodes, 1):
             node_key = f"{node['host']}:{node['port']}"
             stats = node_performance.get(node_key, {})
 
@@ -90,12 +95,14 @@ class AdaptiveParallelismStrategy:
                 is_available = stats.get("available", True)
                 speed_score = 50 if is_available else 1  # Assume available = reasonably fast
 
-            node_speeds.append({
-                "node": node_key,
-                "speed_score": speed_score,
-                "total_requests": total_requests,
-                "is_available": stats.get("available", True)
-            })
+            node_speeds.append(
+                {
+                    "node": node_key,
+                    "speed_score": speed_score,
+                    "total_requests": total_requests,
+                    "is_available": stats.get("available", True),
+                }
+            )
 
         # Sort by speed (fastest first)
         node_speeds.sort(key=lambda x: x["speed_score"], reverse=True)
@@ -125,13 +132,17 @@ class AdaptiveParallelismStrategy:
                 "recommended_node": fastest_node["node"],
             }
 
-        # CASE 2: Small batch (<20 items)
-        if batch_size < 20:
+        # CASE 2: Small batch (dynamic threshold based on node count)
+        # Minimum chunks per node: 5 (so 2 nodes = 10 threshold, 3 nodes = 15, etc.)
+        min_chunks_per_node = 5
+        dynamic_threshold = len(nodes) * min_chunks_per_node
+
+        if batch_size < dynamic_threshold:
             # Overhead of parallelism not worth it
             return False, {
                 **reasoning,
                 "reason": "small_batch",
-                "detail": f"Batch size {batch_size} too small for parallel overhead",
+                "detail": f"Batch size {batch_size} too small for parallel overhead (need {dynamic_threshold}+ for {len(nodes)} nodes)",
                 "recommended_node": fastest_node["node"],
             }
 
@@ -177,8 +188,15 @@ class AdaptiveParallelismStrategy:
 
         nodes = self.pool.nodes
 
-        # Base workers on number of nodes
-        base_workers = len(nodes) * 2
+        # IMPORTANT: Use conservative worker counts to avoid overwhelming Ollama servers
+        # Ollama embedding endpoints can handle multiple concurrent requests efficiently
+        # Multiple workers per node allows better saturation and throughput
+
+        # For small node counts (1-3), use 2x workers to saturate endpoints
+        if len(nodes) <= 3:
+            base_workers = len(nodes) * 2  # 2 workers per node for good saturation
+        else:
+            base_workers = len(nodes) * 2  # Can use more for larger clusters
 
         # Adjust for batch size
         if batch_size < 50:
@@ -186,11 +204,11 @@ class AdaptiveParallelismStrategy:
         elif batch_size < 200:
             workers = base_workers
         else:
-            # Large batch, can use more workers
-            workers = min(base_workers * 2, batch_size)
+            # Large batch, can use slightly more workers
+            workers = min(base_workers + 2, batch_size)  # Conservative increase
 
-        # Minimum 1 worker
-        return max(1, workers)
+        # Minimum 1 worker, maximum 6 (even for large batches)
+        return max(1, min(workers, 6))
 
     def get_recommended_node(self, model_name: Optional[str] = None) -> Optional[str]:
         """
@@ -238,14 +256,16 @@ class AdaptiveParallelismStrategy:
             # Try to detect GPU capability from stats or node info
             has_gpu = node.get("has_gpu", False)
 
-            node_info.append({
-                "key": node_key,
-                "node": node,
-                "avg_latency": avg_latency,
-                "has_gpu": has_gpu,
-                "is_available": is_available,
-                "successful_requests": successful_requests
-            })
+            node_info.append(
+                {
+                    "key": node_key,
+                    "node": node,
+                    "avg_latency": avg_latency,
+                    "has_gpu": has_gpu,
+                    "is_available": is_available,
+                    "successful_requests": successful_requests,
+                }
+            )
 
         # Filter to available nodes only
         available_nodes = [n for n in node_info if n["is_available"]]
@@ -262,12 +282,16 @@ class AdaptiveParallelismStrategy:
         if gpu_nodes:
             # Return fastest GPU node (lowest avg latency)
             fastest_gpu = min(gpu_nodes, key=lambda n: n["avg_latency"])
-            logger.debug(f"Selected GPU node: {fastest_gpu['key']} (latency: {fastest_gpu['avg_latency']:.3f}s)")
+            logger.debug(
+                f"Selected GPU node: {fastest_gpu['key']} (latency: {fastest_gpu['avg_latency']:.3f}s)"
+            )
             return fastest_gpu["key"]
 
         # Fallback to fastest CPU node
         fastest_cpu = min(available_nodes, key=lambda n: n["avg_latency"])
-        logger.debug(f"Selected CPU node: {fastest_cpu['key']} (latency: {fastest_cpu['avg_latency']:.3f}s)")
+        logger.debug(
+            f"Selected CPU node: {fastest_cpu['key']} (latency: {fastest_cpu['avg_latency']:.3f}s)"
+        )
         return fastest_cpu["key"]
 
     def print_parallelism_report(self, batch_size: int):
@@ -347,7 +371,7 @@ def print_parallelism_report(pool):
     Args:
         pool: OllamaPool instance with parallelism strategy
     """
-    if not hasattr(pool, '_parallelism_strategy'):
+    if not hasattr(pool, "_parallelism_strategy"):
         print("⚠️  Adaptive parallelism not enabled on this pool")
         return
 
@@ -357,7 +381,7 @@ def print_parallelism_report(pool):
     print("=" * 60)
 
     # Get metrics from strategy
-    if hasattr(strategy, 'metrics') and strategy.metrics:
+    if hasattr(strategy, "metrics") and strategy.metrics:
         metrics = strategy.metrics
         print(f"\n📊 Decision Metrics:")
         print(f"   Sequential calls: {metrics.get('sequential_count', 0)}")
@@ -372,7 +396,7 @@ def print_parallelism_report(pool):
     print(f"   Min parallel:      {strategy.min_parallel_requests}")
 
     # Show recent decisions
-    if hasattr(strategy, 'recent_decisions'):
+    if hasattr(strategy, "recent_decisions"):
         print(f"\n📝 Recent Decisions:")
         for decision in strategy.recent_decisions[-5:]:
             print(f"   {decision['timestamp']}: {decision['mode']} - {decision['reason']}")
