@@ -10,6 +10,7 @@ This module provides advanced routing decisions based on:
 """
 
 import logging
+import os
 import re
 from dataclasses import dataclass
 from datetime import datetime
@@ -17,10 +18,15 @@ from typing import Dict, List, Optional, Tuple
 
 logger = logging.getLogger(__name__)
 
+# Configurable VRAM threshold for GPU overload detection (in MB)
+# Set via environment variable SOLLOL_MIN_VRAM_MB or defaults to 1000 MB
+MIN_VRAM_THRESHOLD_MB = int(os.environ.get("SOLLOL_MIN_VRAM_MB", "1000"))
+
 # Import VRAM-aware routing utilities
 try:
     from sollol.intelligent_gpu_router import IntelligentGPURouter
-    from sollol.model_sizes import estimate_model_size, can_fit_in_vram
+    from sollol.model_sizes import can_fit_in_vram, estimate_model_size
+
     VRAM_ROUTING_AVAILABLE = True
 except ImportError:
     VRAM_ROUTING_AVAILABLE = False
@@ -188,13 +194,17 @@ class IntelligentRouter:
         complexity, tokens = self.estimate_complexity(payload)
 
         # Determine if GPU is beneficial
-        requires_gpu = task_type in [
-            "generation",
-            "embedding",
-            "summarization",
-            "analysis",
-            "extraction",  # PDF extraction benefits from GPU
-        ] and complexity in ["medium", "complex"]
+        # Embeddings ALWAYS benefit from GPU (vector operations)
+        # Other tasks benefit from GPU for medium/complex workloads
+        if task_type == "embedding":
+            requires_gpu = True  # Always use GPU for embeddings
+        else:
+            requires_gpu = task_type in [
+                "generation",
+                "summarization",
+                "analysis",
+                "extraction",  # PDF extraction benefits from GPU
+            ] and complexity in ["medium", "complex"]
 
         # Estimate duration based on historical data
         estimated_duration = self._estimate_duration(task_type, tokens)
@@ -229,7 +239,7 @@ class IntelligentRouter:
         4. Immediately updates global active request count
 
         GPU Overload Handling:
-        - If all GPUs have low VRAM (<2000 MB), enables CPU fallback
+        - If all GPUs have low VRAM (< MIN_VRAM_THRESHOLD_MB), enables CPU fallback
         - Allows CPU-only nodes to compete when GPUs are overwhelmed
 
         Args:
@@ -248,14 +258,16 @@ class IntelligentRouter:
         # GPU Overload Detection: Check if all GPUs are overwhelmed
         if context.requires_gpu:
             gpu_hosts = [h for h in available if h.get("gpu_free_mem", 0) > 0]
-            all_gpus_overwhelmed = all(
-                h.get("gpu_free_mem", 0) < 2000 for h in gpu_hosts
-            ) if gpu_hosts else True
+            all_gpus_overwhelmed = (
+                all(h.get("gpu_free_mem", 0) < MIN_VRAM_THRESHOLD_MB for h in gpu_hosts)
+                if gpu_hosts
+                else True
+            )
 
             if all_gpus_overwhelmed:
                 # All GPUs overwhelmed - enable CPU fallback
                 logger.warning(
-                    f"All GPUs overwhelmed (VRAM < 2000 MB) - enabling CPU fallback for {context.task_type}"
+                    f"All GPUs overwhelmed (VRAM < {MIN_VRAM_THRESHOLD_MB} MB) - enabling CPU fallback for {context.task_type}"
                 )
                 # Modify context to allow CPU fallback
                 context = TaskContext(
@@ -310,7 +322,8 @@ class IntelligentRouter:
                     "complexity": context.complexity,
                     "reasoning": self._explain_decision(best_host, context, scored_hosts),
                     "alternatives": [
-                        {"host": h["host"], "score": s} for h, s in scored_hosts[1:3]  # Top 2 alternatives
+                        {"host": h["host"], "score": s}
+                        for h, s in scored_hosts[1:3]  # Top 2 alternatives
                     ],
                     "distributed": True,
                     "global_active_requests": new_count,
@@ -374,7 +387,7 @@ class IntelligentRouter:
             if gpu_mem == 0:
                 # No GPU but task needs it - heavy penalty
                 score *= 0.2  # Still possible but very low priority
-            elif gpu_mem < 2000:
+            elif gpu_mem < MIN_VRAM_THRESHOLD_MB:
                 # Low VRAM GPU (1050Ti, etc.) - SHOULD BE USED for overflow when others loaded
                 # Light penalty - will get used when high-VRAM GPUs saturate
                 # VRAM-aware active_requests penalty (50% per request) prevents overload
@@ -394,8 +407,8 @@ class IntelligentRouter:
             elif gpu_mem > 4000:
                 # Good GPU availability - bonus! (3060, 3070, 4060, etc.)
                 score *= 1.5
-            elif gpu_mem >= 2000:
-                # Mid-range VRAM (2-4GB) - neutral
+            elif gpu_mem >= MIN_VRAM_THRESHOLD_MB:
+                # Mid-range VRAM (threshold to 4GB) - neutral
                 score *= 1.0
         elif context.metadata.get("gpu_fallback_to_cpu"):
             # GPU fallback to CPU enabled - prefer hosts with available resources
@@ -404,7 +417,7 @@ class IntelligentRouter:
             if gpu_mem == 0:
                 # CPU-only node - acceptable when GPU fallback enabled
                 score *= 1.0  # No penalty (fair chance against overwhelmed GPUs)
-            elif gpu_mem < 2000:
+            elif gpu_mem < MIN_VRAM_THRESHOLD_MB:
                 # Overwhelmed GPU - slight penalty
                 score *= 0.7
             else:
@@ -414,7 +427,7 @@ class IntelligentRouter:
         # CPU requirements based on complexity
         # Skip CPU penalties for low-VRAM overflow GPUs - any GPU is better than waiting
         gpu_mem = host_meta.get("gpu_free_mem", 4000)
-        is_overflow_gpu = gpu_mem < 2000  # Low-VRAM GPU used for overflow
+        is_overflow_gpu = gpu_mem < MIN_VRAM_THRESHOLD_MB  # Low-VRAM GPU used for overflow
 
         cpu_load = host_meta.get("cpu_load", 0.5)  # Get cpu_load for all hosts
 
@@ -455,30 +468,56 @@ class IntelligentRouter:
             score /= 1 + latency_penalty
 
         # Factor 4: Additional load considerations
-        # Active concurrent requests - VRAM-aware load balancing
-        # Penalty proportional to VRAM capacity to avoid overloading small GPUs
+        # Active concurrent requests - BALANCED exponential penalty
+        # Goal: Prioritize SPEED (fastest completion) over forced distribution
+        # A powerful GPU with 1 request may still be faster than weak GPU with 0 requests
         active_requests = host_meta.get("active_requests", 0)
         if active_requests > 0:
             gpu_mem = host_meta.get("gpu_free_mem", 4000)  # Default to mid-range
+            avg_latency = host_meta.get(
+                "latency_ms", MIN_VRAM_THRESHOLD_MB * 2
+            )  # Historical avg latency
 
-            # Calculate penalty based on VRAM capacity
-            # Small GPUs get penalized more heavily per request (avoid overload)
-            # Large GPUs can handle more concurrent requests
-            if gpu_mem < 2000:
-                # Small GPU (1050Ti) - each request is significant load
-                penalty_per_request = 0.50  # 50% penalty per request (aggressive)
+            # Calculate LIGHTER exponential penalty based on GPU capability
+            # Key insight: Powerful GPUs can handle multiple requests efficiently
+            # Small GPUs saturate quickly, large GPUs maintain performance
+            if gpu_mem < MIN_VRAM_THRESHOLD_MB:
+                # Small GPU (1050Ti) - saturates quickly
+                # 0 req: 1.0x, 1 req: 0.50x, 2 req: 0.33x, 3 req: 0.25x
+                # Still usable with multiple requests, just slower
+                exponential_base = 1.5  # Reduced from 3.0
             elif gpu_mem < 4000:
-                # Mid-small GPU (2-4GB) - moderate-high penalty
-                penalty_per_request = 0.35  # 35% penalty per request
+                # Mid-small GPU (2-4GB)
+                # 0 req: 1.0x, 1 req: 0.59x, 2 req: 0.42x, 3 req: 0.31x
+                exponential_base = 1.3  # Reduced from 2.5
             elif gpu_mem < 8000:
-                # Mid GPU (4-8GB, like 3060) - moderate penalty
-                penalty_per_request = 0.25  # 25% penalty per request
+                # Mid GPU (4-8GB, like 3060)
+                # 0 req: 1.0x, 1 req: 0.71x, 2 req: 0.56x, 3 req: 0.45x
+                exponential_base = 1.2  # Reduced from 2.0
             else:
-                # Large GPU (8GB+, like 4090/5090) - light penalty
-                penalty_per_request = 0.15  # 15% penalty per request (can handle load)
+                # Large GPU (8GB+, like 4090/5090) - minimal saturation
+                # 0 req: 1.0x, 1 req: 0.83x, 2 req: 0.71x, 3 req: 0.62x
+                # Powerful GPUs maintain good performance with multiple requests
+                exponential_base = 1.15  # Reduced from 1.5
 
-            active_penalty = active_requests * penalty_per_request
-            score /= 1 + active_penalty
+            # PERFORMANCE-AWARE: Adjust penalty based on actual latency
+            # Fast nodes (low latency) get reduced penalty - they're efficient
+            # Slow nodes (high latency) get increased penalty - already struggling
+            if avg_latency < 1000:  # Fast node (<1s avg)
+                # Reduce penalty for fast nodes - they can handle more
+                exponential_base = max(1.05, exponential_base * 0.8)
+            elif avg_latency > 5000:  # Slow node (>5s avg)
+                # Increase penalty for slow nodes - already overloaded
+                exponential_base = min(2.0, exponential_base * 1.3)
+
+            # EXPONENTIAL penalty: score / (base ^ requests)
+            # Less aggressive than before - allows powerful nodes to stay competitive
+            score /= exponential_base**active_requests
+
+            # LINEAR penalty for fairness - combined with exponential
+            # This provides base load balancing without being too aggressive
+            linear_penalty = 0.15 * active_requests  # 15% per request
+            score /= 1 + linear_penalty
 
         # CPU load - historical average
         # Penalize heavily loaded nodes more for high-priority tasks
@@ -488,7 +527,30 @@ class IntelligentRouter:
             load_penalty = cpu_load * 1.5  # Standard penalty
         score /= 1 + load_penalty
 
-        # Factor 5: Priority alignment
+        # Factor 5: Model warmth (avoid cold loads)
+        # CRITICAL: Give huge bonus to nodes with model already loaded (avoid 18+ sec cold loads)
+        if context.model_preference:
+            loaded_models = host_meta.get("loaded_models", [])
+            # Check if target model is loaded (handle both "model" and "model:tag" formats)
+            model_base = context.model_preference.split(":")[0]
+            is_loaded = any(
+                context.model_preference == loaded
+                or model_base in loaded
+                or loaded.startswith(model_base)
+                for loaded in loaded_models
+            )
+            if is_loaded:
+                # Model already loaded - moderate bonus (1.5x) to avoid cold loads
+                # Reduced from 3x to allow GPU capability to matter more
+                score *= 1.5
+                logger.debug(
+                    f"   ✅ Model {context.model_preference} already loaded on {host_meta.get('host', 'unknown')} - 1.5x bonus"
+                )
+            elif loaded_models:  # Has other models loaded
+                # Not loaded - slight penalty for nodes with full VRAM
+                score *= 0.95
+
+        # Factor 6: Priority alignment
         # Prefer priority 0 hosts for high-priority tasks
         host_priority = host_meta.get("priority", 999)
         if host_priority == 0 and context.priority >= 7:
